@@ -1,15 +1,14 @@
-from decimal import Decimal
 from numbers import Real
 from typing import (List,
-                    Tuple,
-                    TypeVar)
+                    Tuple)
 
 from hypothesis.strategies import SearchStrategy
 
 Strategy = SearchStrategy
-Coordinate = TypeVar('Coordinate', Real, Decimal)
+Coordinate = Real
 Point = Tuple[Coordinate, Coordinate]
 Segment = Tuple[Point, Point]
+BoundingBox = Tuple[Coordinate, Coordinate, Coordinate, Coordinate]
 Contour = List[Point]
 Polyline = List[Point]
 Polygon = Tuple[Contour, List[Contour]]
