@@ -3,9 +3,9 @@ from functools import partial
 from itertools import (cycle,
                        groupby,
                        repeat)
+from random import Random
 from typing import (List,
                     Optional,
-                    Sequence,
                     Sized,
                     Tuple)
 
@@ -14,9 +14,9 @@ from hypothesis.errors import HypothesisWarning
 
 from .core.contracts import (is_contour_non_convex,
                              is_contour_strict,
-                             is_non_self_intersecting_contour,
                              points_do_not_lie_on_the_same_line)
-from .hints import (Contour,
+from .hints import (BoundingBox,
+                    Contour,
                     Coordinate,
                     Point,
                     Polygon,
@@ -25,6 +25,7 @@ from .hints import (Contour,
                     Strategy)
 from .utils import (constrict_convex_hull_size,
                     pack,
+                    sort_pair,
                     to_concave_contour,
                     to_convex_contour,
                     to_convex_hull,
@@ -302,14 +303,14 @@ def _polylines(x_coordinates: Strategy[Coordinate],
                             max_size=max_size)))
 
 
-TRIANGLE_SIZE = 3
+TRIANGULAR_CONTOUR_SIZE = 3
 MIN_CONCAVE_CONTOUR_SIZE = 4
 
 
 def contours(x_coordinates: Strategy[Coordinate],
              y_coordinates: Optional[Strategy[Coordinate]] = None,
              *,
-             min_size: int = TRIANGLE_SIZE,
+             min_size: int = TRIANGULAR_CONTOUR_SIZE,
              max_size: Optional[int] = None) -> Strategy[Contour]:
     """
     Returns a strategy for contours.
@@ -390,10 +391,10 @@ def contours(x_coordinates: Strategy[Coordinate],
     ...     for vertex_x, vertex_y in contour)
     True
     """
-    _validate_sizes(min_size, max_size, TRIANGLE_SIZE)
-    if max_size is not None and max_size == TRIANGLE_SIZE:
+    _validate_sizes(min_size, max_size, TRIANGULAR_CONTOUR_SIZE)
+    if max_size is not None and max_size == TRIANGULAR_CONTOUR_SIZE:
         return triangular_contours(x_coordinates, y_coordinates)
-    min_size = max(min_size, TRIANGLE_SIZE)
+    min_size = max(min_size, TRIANGULAR_CONTOUR_SIZE)
     return (convex_contours(x_coordinates, y_coordinates,
                             min_size=min_size,
                             max_size=max_size)
@@ -406,7 +407,7 @@ def contours(x_coordinates: Strategy[Coordinate],
 def convex_contours(x_coordinates: Strategy[Coordinate],
                     y_coordinates: Optional[Strategy[Coordinate]] = None,
                     *,
-                    min_size: int = TRIANGLE_SIZE,
+                    min_size: int = TRIANGULAR_CONTOUR_SIZE,
                     max_size: Optional[int] = None) -> Strategy[Contour]:
     """
     Returns a strategy for convex contours.
@@ -486,33 +487,29 @@ def convex_contours(x_coordinates: Strategy[Coordinate],
     ...     for vertex_x, vertex_y in contour)
     True
     """
-    _validate_sizes(min_size, max_size, TRIANGLE_SIZE)
-    if max_size is not None and max_size == TRIANGLE_SIZE:
+    _validate_sizes(min_size, max_size, TRIANGULAR_CONTOUR_SIZE)
+    if max_size is not None and max_size == TRIANGULAR_CONTOUR_SIZE:
         return triangular_contours(x_coordinates, y_coordinates)
-    min_size = max(min_size, TRIANGLE_SIZE)
+    min_size = max(min_size, TRIANGULAR_CONTOUR_SIZE)
 
-    def to_points_with_flags_and_permutations(
-            points: List[Point]) -> Strategy[Tuple[List[Point],
-                                                   List[bool], List[bool],
-                                                   Sequence[int]]]:
-        flags = strategies.lists(strategies.booleans(),
-                                 min_size=len(points),
-                                 max_size=len(points))
-        return strategies.tuples(strategies.just(points),
-                                 flags, flags,
-                                 strategies.permutations(range(len(points))))
+    def to_points_with_random(points: List[Point]
+                              ) -> Strategy[Tuple[List[Point], Random]]:
+        return strategies.tuples(strategies.just(points), strategies.randoms())
 
     result = (strategies.lists(points(x_coordinates, y_coordinates),
                                min_size=min_size,
                                max_size=max_size,
                                unique=True)
-              .flatmap(to_points_with_flags_and_permutations)
+              .flatmap(to_points_with_random)
               .map(pack(to_convex_contour))
               .filter(partial(_has_valid_size,
                               min_size=min_size,
                               max_size=max_size)))
+    result = (rectangular_contours(x_coordinates, y_coordinates) | result
+              if min_size <= RECTANGULAR_CONTOUR_SIZE
+              else result)
     return (triangular_contours(x_coordinates, y_coordinates) | result
-            if min_size <= TRIANGLE_SIZE
+            if min_size == TRIANGULAR_CONTOUR_SIZE
             else result)
 
 
@@ -599,17 +596,26 @@ def concave_contours(x_coordinates: Strategy[Coordinate],
     """
     _validate_sizes(min_size, max_size, MIN_CONCAVE_CONTOUR_SIZE)
     min_size = max(min_size, MIN_CONCAVE_CONTOUR_SIZE)
+
+    def to_points_with_sizes(points: List[Point]
+                             ) -> Strategy[Tuple[List[Point], int]]:
+        sizes = strategies.integers(min_size,
+                                    len(points)
+                                    if max_size is None
+                                    else min(len(points), max_size))
+        return strategies.tuples(strategies.just(points), sizes)
+
     return (strategies.lists(points(x_coordinates, y_coordinates),
                              min_size=min_size,
                              max_size=max_size,
                              unique=True)
             .filter(points_do_not_lie_on_the_same_line)
-            .map(to_concave_contour)
+            .flatmap(to_points_with_sizes)
+            .map(pack(to_concave_contour))
             .filter(partial(_has_valid_size,
                             min_size=min_size,
                             max_size=max_size))
-            .filter(is_contour_non_convex)
-            .filter(is_non_self_intersecting_contour))
+            .filter(is_contour_non_convex))
 
 
 def triangular_contours(x_coordinates: Strategy[Coordinate],
@@ -689,17 +695,185 @@ def triangular_contours(x_coordinates: Strategy[Coordinate],
             .map(list))
 
 
+RECTANGULAR_CONTOUR_SIZE = 4
+
+
+def rectangular_contours(x_coordinates: Strategy[Coordinate],
+                         y_coordinates: Optional[Strategy[Coordinate]] = None,
+                         ) -> Strategy[Contour]:
+    """
+    Returns a strategy for axis-aligned rectangular contours.
+    Rectangular contour is a contour formed by 4 points.
+
+    :param x_coordinates: strategy for vertices' x-coordinates.
+    :param y_coordinates:
+        strategy for vertices' y-coordinates,
+        ``None`` for reusing x-coordinates strategy.
+
+    >>> from hypothesis import strategies
+    >>> from hypothesis_geometry import planar
+
+    For same coordinates' domain:
+
+    >>> min_coordinate, max_coordinate = -1., 1.
+    >>> coordinates_type = float
+    >>> coordinates = strategies.floats(min_coordinate, max_coordinate,
+    ...                                 allow_infinity=False,
+    ...                                 allow_nan=False)
+    >>> contours = planar.rectangular_contours(coordinates)
+    >>> contour = contours.example()
+    >>> isinstance(contour, list)
+    True
+    >>> len(contour) == 4
+    True
+    >>> all(isinstance(vertex, tuple) for vertex in contour)
+    True
+    >>> all(len(vertex) == 2 for vertex in contour)
+    True
+    >>> all(all(isinstance(coordinate, coordinates_type)
+    ...         for coordinate in vertex)
+    ...     for vertex in contour)
+    True
+    >>> all(all(min_coordinate <= coordinate <= max_coordinate
+    ...         for coordinate in vertex)
+    ...     for vertex in contour)
+    True
+
+    For different coordinates' domains:
+
+    >>> min_x_coordinate, max_x_coordinate = -1., 1.
+    >>> min_y_coordinate, max_y_coordinate = 10., 100.
+    >>> coordinates_type = float
+    >>> x_coordinates = strategies.floats(min_x_coordinate, max_x_coordinate,
+    ...                                   allow_infinity=False,
+    ...                                   allow_nan=False)
+    >>> y_coordinates = strategies.floats(min_y_coordinate, max_y_coordinate,
+    ...                                   allow_infinity=False,
+    ...                                   allow_nan=False)
+    >>> contours = planar.rectangular_contours(x_coordinates, y_coordinates)
+    >>> contour = contours.example()
+    >>> isinstance(contour, list)
+    True
+    >>> len(contour) == 4
+    True
+    >>> all(isinstance(vertex, tuple) for vertex in contour)
+    True
+    >>> all(len(vertex) == 2 for vertex in contour)
+    True
+    >>> all(all(isinstance(coordinate, coordinates_type)
+    ...         for coordinate in vertex)
+    ...     for vertex in contour)
+    True
+    >>> all(min_x_coordinate <= vertex_x <= max_x_coordinate
+    ...     and min_y_coordinate <= vertex_y <= max_y_coordinate
+    ...     for vertex_x, vertex_y in contour)
+    True
+    """
+
+    def bounding_box_to_contour(bounding_box: BoundingBox) -> Contour:
+        x_min, x_max, y_min, y_max = bounding_box
+        return [(x_min, y_min), (x_max, y_min), (x_max, y_max), (x_min, y_max)]
+
+    return (bounding_boxes(x_coordinates, y_coordinates)
+            .map(bounding_box_to_contour))
+
+
+def bounding_boxes(x_coordinates: Strategy[Coordinate],
+                   y_coordinates: Optional[Strategy[Coordinate]] = None,
+                   ) -> Strategy[BoundingBox]:
+    """
+    Returns a strategy for bounding boxes.
+    Bounding box is a quadruple of ``x_min``, ``x_max``, ``y_min``, ``y_max``.
+
+    :param x_coordinates: strategy for vertices' x-coordinates.
+    :param y_coordinates:
+        strategy for vertices' y-coordinates,
+        ``None`` for reusing x-coordinates strategy.
+
+    >>> from hypothesis import strategies
+    >>> from hypothesis_geometry import planar
+
+    For same coordinates' domain:
+
+    >>> min_coordinate, max_coordinate = -1., 1.
+    >>> coordinates_type = float
+    >>> coordinates = strategies.floats(min_coordinate, max_coordinate,
+    ...                                 allow_infinity=False,
+    ...                                 allow_nan=False)
+    >>> bounding_boxes = planar.bounding_boxes(coordinates)
+    >>> bounding_box = bounding_boxes.example()
+    >>> isinstance(bounding_box, tuple)
+    True
+    >>> len(bounding_box) == 4
+    True
+    >>> all(isinstance(coordinate, coordinates_type)
+    ...     for coordinate in bounding_box)
+    True
+    >>> all(min_coordinate <= coordinate <= max_coordinate
+    ...     for coordinate in bounding_box)
+    True
+
+    For different coordinates' domains:
+
+    >>> min_x_coordinate, max_x_coordinate = -1., 1.
+    >>> min_y_coordinate, max_y_coordinate = 10., 100.
+    >>> coordinates_type = float
+    >>> x_coordinates = strategies.floats(min_x_coordinate, max_x_coordinate,
+    ...                                   allow_infinity=False,
+    ...                                   allow_nan=False)
+    >>> y_coordinates = strategies.floats(min_y_coordinate, max_y_coordinate,
+    ...                                   allow_infinity=False,
+    ...                                   allow_nan=False)
+    >>> bounding_boxes = planar.bounding_boxes(x_coordinates, y_coordinates)
+    >>> bounding_box = bounding_boxes.example()
+    >>> isinstance(bounding_box, tuple)
+    True
+    >>> len(bounding_box) == 4
+    True
+    >>> all(isinstance(coordinate, coordinates_type)
+    ...     for coordinate in bounding_box)
+    True
+    >>> all(min_x_coordinate <= coordinate <= max_x_coordinate
+    ...     for coordinate in bounding_box[:2])
+    True
+    >>> all(min_y_coordinate <= coordinate <= max_y_coordinate
+    ...     for coordinate in bounding_box[2:])
+    True
+    """
+    if y_coordinates is None:
+        y_coordinates = x_coordinates
+
+    def to_bounding_box(x_bounds: Tuple[Coordinate, Coordinate],
+                        y_bounds: Tuple[Coordinate, Coordinate]
+                        ) -> BoundingBox:
+        x_min, x_max = x_bounds
+        y_min, y_max = y_bounds
+        return x_min, x_max, y_min, y_max
+
+    return strategies.builds(to_bounding_box,
+                             strategies.lists(x_coordinates,
+                                              min_size=2,
+                                              max_size=2,
+                                              unique=True)
+                             .map(sort_pair),
+                             strategies.lists(y_coordinates,
+                                              min_size=2,
+                                              max_size=2,
+                                              unique=True)
+                             .map(sort_pair))
+
+
 EMPTY_HOLES_SIZE = 0
 
 
 def polygons(x_coordinates: Strategy[Coordinate],
              y_coordinates: Optional[Strategy[Coordinate]] = None,
              *,
-             min_size: int = TRIANGLE_SIZE,
+             min_size: int = TRIANGULAR_CONTOUR_SIZE,
              max_size: Optional[int] = None,
              min_holes_size: int = EMPTY_HOLES_SIZE,
              max_holes_size: Optional[int] = None,
-             min_hole_size: int = TRIANGLE_SIZE,
+             min_hole_size: int = TRIANGULAR_CONTOUR_SIZE,
              max_hole_size: Optional[int] = None) -> Strategy[Polygon]:
     """
     Returns a strategy for polygons.
@@ -849,13 +1023,13 @@ def polygons(x_coordinates: Strategy[Coordinate],
     ...     for vertex_x, vertex_y in hole)
     True
     """
-    _validate_sizes(min_size, max_size, TRIANGLE_SIZE)
+    _validate_sizes(min_size, max_size, TRIANGULAR_CONTOUR_SIZE)
     _validate_sizes(min_holes_size, max_holes_size, EMPTY_HOLES_SIZE,
                     'min_holes_size', 'max_holes_size')
-    _validate_sizes(min_hole_size, max_hole_size, TRIANGLE_SIZE,
+    _validate_sizes(min_hole_size, max_hole_size, TRIANGULAR_CONTOUR_SIZE,
                     'min_hole_size', 'max_hole_size')
-    min_size, min_hole_size = (max(min_size, TRIANGLE_SIZE),
-                               max(min_hole_size, TRIANGLE_SIZE))
+    min_size, min_hole_size = (max(min_size, TRIANGULAR_CONTOUR_SIZE),
+                               max(min_hole_size, TRIANGULAR_CONTOUR_SIZE))
 
     def to_points_with_sizes(points: List[Point]
                              ) -> Strategy[Tuple[List[Point], int, List[int]]]:
