@@ -1,5 +1,6 @@
 from functools import partial
 from math import atan2
+from random import Random
 from typing import (Callable,
                     Iterable,
                     List,
@@ -15,14 +16,14 @@ from .core import triangular
 from .core.subdivisional import (QuadEdge,
                                  to_edge_neighbours)
 from .core.utils import (Orientation,
-                         to_orientation)
+                         orientation)
 from .hints import (Contour,
                     Coordinate,
                     Point,
                     Polygon)
 
 
-def to_concave_contour(points: Sequence[Point]) -> Contour:
+def to_concave_contour(points: Sequence[Point], size: int) -> Contour:
     """
     Based on chi-algorithm by M. Duckham et al.
 
@@ -44,10 +45,15 @@ def to_concave_contour(points: Sequence[Point]) -> Contour:
                         for edge in boundary_edges}
     candidates = red_black.tree(*filter(is_mouth, boundary_edges),
                                 key=_edge_key)
-    while candidates:
-        edge = candidates.popmax()
+    current_size = len(to_strict_convex_hull(points))
+    while current_size < size:
+        try:
+            edge = candidates.popmax()
+        except KeyError:
+            break
         if not is_mouth(edge):
             continue
+        size += 1
         boundary_vertices.add(edge.left_from_start.end)
         triangulation.delete(edge)
         for neighbour in edges_neighbours.pop(edge):
@@ -70,7 +76,7 @@ def to_polygon(points: Sequence[Point],
     holes = []
     for hole_size in holes_sizes:
         hole_points = inner_points[start:start + hole_size]
-        holes.append(to_concave_contour(hole_points))
+        holes.append(to_concave_contour(hole_points, hole_size))
         boundary_vertices.update(hole_points)
         start += hole_size
 
@@ -138,7 +144,7 @@ def constrict_convex_hull_size(points: List[Point],
     return (new_border
             + [point
                for point in set(points) - set(convex_hull)
-               if all(to_orientation(end, start, point)
+               if all(orientation(end, start, point)
                       is Orientation.COUNTERCLOCKWISE
                       for start, end in new_border_extra_segments)])
 
@@ -149,9 +155,7 @@ def _to_squared_points_distance(left: Point, right: Point) -> Coordinate:
 
 
 def to_convex_contour(points: List[Point],
-                      x_flags: List[bool],
-                      y_flags: List[bool],
-                      permutation: Sequence[int]) -> Contour:
+                      random: Random) -> Contour:
     """
     Based on Valtr algorithm by Sander Verdonschot.
 
@@ -168,13 +172,12 @@ def to_convex_contour(points: List[Point],
     min_y, *ys, max_y = ys
 
     def to_vectors_coordinates(coordinates: List[Coordinate],
-                               flags: List[bool],
                                min_coordinate: Coordinate,
                                max_coordinate: Coordinate) -> List[Coordinate]:
         last_min = last_max = min_coordinate
         result = []
-        for flag, coordinate in zip(flags, coordinates):
-            if flag:
+        for coordinate in coordinates:
+            if random.getrandbits(1):
                 result.append(coordinate - last_min)
                 last_min = coordinate
             else:
@@ -183,9 +186,9 @@ def to_convex_contour(points: List[Point],
         result.extend((max_coordinate - last_min, last_max - max_coordinate))
         return result
 
-    vectors_xs = to_vectors_coordinates(xs, x_flags, min_x, max_x)
-    vectors_ys = to_vectors_coordinates(ys, y_flags, min_y, max_y)
-    vectors_ys = [vectors_ys[index] for index in permutation]
+    vectors_xs = to_vectors_coordinates(xs, min_x, max_x)
+    vectors_ys = to_vectors_coordinates(ys, min_y, max_y)
+    random.shuffle(vectors_ys)
 
     def to_vector_angle(vector: Tuple[Coordinate, Coordinate]) -> Sortable:
         x, y = vector
@@ -212,15 +215,15 @@ def shrink_collinear_vertices(contour: Contour) -> None:
     index = -len(contour) + 1
     while index < 0:
         while (max(2, -index) < len(contour)
-               and (to_orientation(contour[index + 2], contour[index + 1],
-                                   contour[index])
+               and (orientation(contour[index + 2], contour[index + 1],
+                                contour[index])
                     is Orientation.COLLINEAR)):
             del contour[index + 1]
         index += 1
     while index < len(contour):
         while (max(2, index) < len(contour)
-               and (to_orientation(contour[index - 2], contour[index - 1],
-                                   contour[index])
+               and (orientation(contour[index - 2], contour[index - 1],
+                                contour[index])
                     is Orientation.COLLINEAR)):
             del contour[index - 1]
         index += 1
@@ -237,7 +240,7 @@ def _to_sub_hull(points: Iterable[Point]) -> List[Point]:
     result = []
     for point in points:
         while len(result) >= 2:
-            if (to_orientation(result[-1], result[-2], point)
+            if (orientation(result[-1], result[-2], point)
                     is Orientation.CLOCKWISE):
                 del result[-1]
             else:
@@ -259,7 +262,7 @@ def _to_strict_sub_hull(points: Iterable[Point]) -> List[Point]:
     result = []
     for point in points:
         while len(result) >= 2:
-            if (to_orientation(result[-1], result[-2], point)
+            if (orientation(result[-1], result[-2], point)
                     is not Orientation.COUNTERCLOCKWISE):
                 del result[-1]
             else:
@@ -280,3 +283,8 @@ def pack(function: Callable[..., Range]
 def apply(function: Callable[..., Range],
           args: Iterable[Domain]) -> Range:
     return function(*args)
+
+
+def sort_pair(pair: Sequence[Domain]) -> Tuple[Domain, Domain]:
+    first, second = pair
+    return (first, second) if first < second else (second, first)
